﻿BINDING_HEADER_KONG = "Kong Automatic UI Hider";
BINDING_NAME_KONG_ENABLE = "Enable UI Fading";
BINDING_NAME_KONG_DISABLE = "Disable UI Fading";
BINDING_NAME_KONG_TOGGLE = "Toggle UI Fading On/Off";
BINDING_NAME_KONG_CONFIG = "Open Config GUI";
local VERSION = "1.6a";

Kong_Frames = {};
Kong_Profiles = {};

local lastMouseFocus = nil
local lastMouseStart = nil

local MinimapParent;

local CANNOT_SAVE = "Nameless frames cannot be saved.";
local UNKNOWN_OPTION = "Unknown option: %q (Type |cFF33FF99/kong %s|r for help).";
local PROFILE_NAME_REQUIRED = "Profile name required.";
local NO_SUCH_PROFILE = "Profile %q does not exist. Use |cFF33FF99/kong profile list|r to see a list of available profiles.";
local PROFILE_LOADED = "Profile %q loaded.";
local NOW_USING_PROFILE = "Now using profile %q.";
local PROFILE_ALREADY_EXISTS = "Profile %q already exists.";
local CANNOT_DELETE_CURRENT_PROFILE = "Cannot delete the current profile. Use |cFF33FF99/kong profile reset|r to reset it.";
local PROFILE_ALREADY_LOADED = "Profile %q is already the current profile.";
local PROFILE_DELETED = "Profile %q deleted.";
local PROFILE_RESET = "Current profile %q has been reset.";
local HELP_PROFILE_COMMANDS = "|cFFBBBBFFKong Automatic UI Hider|r profile commands ( |cFF33FF99/kong profile|r ):";
local HELP_PROFILE_LOAD = "|cFF33FF99load <name>|r: loads the specified profile.";
local HELP_PROFILE_CREATE = "|cFF33FF99create <name>|r: creates a new profile with the given name.";
local HELP_PROFILE_DELETE = "|cFF33FF99delete <name>|r: deletes the specified profile.";
local HELP_PROFILE_LIST = "|cFF33FF99list|r: lists all available profiles.";
local HELP_PROFILE_CURRENT = "|cFF33FF99current|r: prints the current profile name.";
local HELP_PROFILE_RESET = "|cFF33FF99reset|r: resets the current profile.";
local INVALID_COMMAND = " command unknown (Type |cFF33FF99/kong help|r for help)."
local NO_KONG_CONFIG = "Kong Config is either disabled or not installed. Defaulting to command-line interface..."
local HELP_COMMANDS = "|cFFBBBBFFKong Automatic UI Hider|r commands ( |cFF33FF99/kong|r ):";
local HELP_TOGGLE = "|cFF33FF99enable / disable / toggle|r: enables or disables fading of the user interface.";
local HELP_PROFILE = "|cFF33FF99profile|r: manages saved Kong configurations and allows for configuration sharing between characters.";
local WELCOME = "New character detected. Type |cFF33FF99/Kong|r to configure."
local ENABLED = "Enabled.";
local ALREADY_ENABLED = "Already enabled."
local DISABLED = "Disabled.";
local ALREADY_DISABLED = "Already disabled.";

StaticPopupDialogs["KONG_RESET"] = {
    text = "Are you sure you want to delete all settings from the Kong profile \"%s\"?",
    button1 = "Reset",
    button2 = CANCEL,
    OnAccept = function(self)
    
        Kong_Unload();
        Kong_Profiles[Kong_Settings.profile] = nil
        Kong_LoadSettings();
        
        Kong_Info(PROFILE_RESET, Kong_Settings.profile);
    end,
    showAlert = 1,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
}

-- The defaults for all triggers
local TriggerDefaults = {
    secondsIn = .2,
    secondsOut = 1,
    alphaIn = 1,
    alphaOut = 0,
};

local pairs = _G.pairs;
local ipairs = _G.ipairs;

function Kong_Error(str, ...)
    DEFAULT_CHAT_FRAME:AddMessage("|cFFFF2222Kong Error:|r "..string.format(str, ...));
end

function Kong_Warning(str, ...)
    DEFAULT_CHAT_FRAME:AddMessage("|cFFFF9922Kong Warning:|r "..string.format(str, ...));
end

function Kong_Info(str, ...)
    DEFAULT_CHAT_FRAME:AddMessage("|cFF33FF99Kong:|r "..string.format(str, ...));
end

local LibUSC = LibStub("LibUserSpecifiedConditions", true);
if not LibUSC then
    Kong_Error("Kong does not appear to have been installed correctly. Note: A full restart of "
             .."World of Warcraft is required when upgrading from a previous version of Kong.")
    return
end
    
-- Dummy condition
MouseoverTypeName = "Frame has Mouse Focus"
LibUSC:RegisterParameter(MouseoverTypeName, { 
    type = "Truth",
    isHidden = true,
    value = false,
})

function Kong_OverrideAlpha(region, override)
	if ( Kong_Frames[region] == nil ) then return end
	
	if ( override ) then
		region.kongDoOverrideAlphaHook = true
		region.kongFaderAlpha = region:GetAlpha()
		region.kongFrameAlpha = region:GetAlpha()
		region.KongSetAlpha = function(region, alpha)
				region.kongDoOverrideAlphaHook = false
				region.kongFaderAlpha = alpha
				region:SetAlpha(math.min(region.kongFrameAlpha, alpha))
				region.kongDoOverrideAlphaHook = true
		end
		
		if ( not region.kongAlphaOverridden ) then
			hooksecurefunc(region, "SetAlpha", function(self, alpha)
					if ( self.kongDoOverrideAlphaHook ) then
						self.kongDoOverrideAlphaHook = false
						self.kongFrameAlpha = alpha
						self:SetAlpha(math.min(alpha, self.kongFaderAlpha))
						self.kongDoOverrideAlphaHook = true
					end
			end)
			region.kongAlphaOverridden = true
		end
	else
		region.kongDoOverrideAlphaHook = nil
		region.kongFaderAlpha = nil
		region.kongFrameAlpha = nil
		region.KongSetAlpha = region.SetAlpha
	end
end

function Kong_SetFader(region, compatibility)
	assert(Kong_Frames[region] ~= nil)
	region.KongAlphaGroup = region.KongAlphaGroup or region:CreateAnimationGroup()
	
	local animation = region.KongAlphaGroup:GetAnimations()
	if ( animation == nil ) then
		if ( compatibility ) then
			local animation = region.KongAlphaGroup:CreateAnimation("animation")
			function animation:SetChange(change)
				self.change = change
			end
			animation:SetScript("OnPlay", function(self)
					self.startAlpha = region:KongGetAlpha()
			end)
			animation:SetScript("OnUpdate", function(self, elapsed)
					region:SetAlpha(self.startAlpha + (self.change * self:GetSmoothProgress()))
			end)
		else
			local animation = region.KongAlphaGroup:CreateAnimation("alpha")
			animation:SetScript("OnPlay", function(self)
				self.startAlpha = region:KongGetAlpha()
			end)
			animation:SetScript("OnUpdate", function() end)
		end
	end
	Kong_Frames[region].compatibilityFader = compatibility
end

function Kong_CallOnFrameHeirarchy(regions, funcName, ...)
	for i, region in pairs(regions) do
		region[funcName](region, ...)
		Kong_CallOnFrameHeirarchy({region:GetChildren()}, funcName, ...)
	end
end

function Kong_DisableDraw(Frame)
    local Frames = {Frame}
    Kong_CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "BACKGROUND")
    Kong_CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "BORDER")
    Kong_CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "ARTWORK")
    Kong_CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "OVERLAY")
end

function Kong_EnableDraw(Frame) 
    local Frames = {Frame}
    Kong_CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "BACKGROUND")
    Kong_CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "BORDER")
    Kong_CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "ARTWORK")
    Kong_CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "OVERLAY")
    
end
  
function Kong_Fade(region, startAlpha, endAlpha, duration, finishFunc)
	region.KongAlphaGroup:Stop()
	
	if ( Kong_Settings.enabled and (KongConfig == nil or not KongConfig:IsShown()) ) then
	
		region:KongSetAlpha(startAlpha)
		local animation = region.KongAlphaGroup:GetAnimations()
		if ( animation:IsObjectType("alpha") ) then
			animation:SetFromAlpha(startAlpha)
			animation:SetToAlpha(endAlpha)
		else
			animation:SetChange(endAlpha - startAlpha)
		end
		animation:SetDuration(duration)
		
		local function onFinish(self)
			region:KongSetAlpha(endAlpha)
			if ( finishFunc ~= nil ) then
				finishFunc(region)
			end
		end
		
		region.KongAlphaGroup:SetScript("OnFinished", onFinish)
		region.KongAlphaGroup:SetScript("OnStop", onFinish)
		region.KongAlphaGroup:Play()
		
		if ( duration == 0 ) then
			region.KongAlphaGroup:Stop()
		end
	end
end

function Kong_FadeIn(region, condition)
	condition.refs = condition.refs + 1
	if ( condition.refs == 1 ) then
		Kong_CalculateFadeIn(region)
	end
end

function Kong_CalculateFadeIn(region)
	local startAlpha = ceil(region:KongGetAlpha() * 100) / 100
	local endAlpha = 1
	local fastestRate = 0
	local finishFunc = nil
	
	for i, condition in ipairs(Kong_Frames[region]) do
		if ( condition.refs > 0 and condition.alphaIn >= startAlpha ) then
			if ( condition.alphaIn > startAlpha ) then
				finishFunc = Kong_CalculateFadeIn
			end
			local rate = (condition.alphaIn - condition.alphaOut) / condition.secondsIn
			if ( rate > fastestRate or (rate == fastestRate and condition.alphaIn > endAlpha) ) then
				endAlpha = condition.alphaIn
				fastestRate = rate
			end
		end
	end
	
	if ( fastestRate > 0 ) then
		local duration = (endAlpha - startAlpha) / fastestRate
		Kong_Fade(region, startAlpha, endAlpha, duration, finishFunc)
		if ( Kong_Frames[region].disableDrawWhenHidden and startAlpha == 0 ) then
			Kong_EnableDraw(region)
		end
	end
end

function Kong_FadeOut(region, condition)
	condition.refs = condition.refs - 1
	assert(condition.refs >= 0, "ref < 0 for region: " .. tostring(region:GetName() or region))
	if ( condition.refs == 0 ) then
		local rate = (condition.alphaOut - condition.alphaIn) / condition.secondsOut
		local displacement = (condition.alphaIn - region:KongGetAlpha()) / rate
		condition.lastFadeTime = GetTime() + displacement
		Kong_CalculateFadeOut(region)
	end
end

function Kong_CalculateFadeOut(frame)
	local startAlpha = frame:KongGetAlpha()
	local endAlpha = Kong_Frames[frame].TriggerDefaults.alphaOut
	local duration = 0
	local finishedFunc
	local chosenTrigger
	
	for i, trigger in ipairs(Kong_Frames[frame]) do
		if ( trigger.refs > 0 ) then
			if ( trigger.alphaIn >= startAlpha ) then return end
			
			endAlpha = math.max(endAlpha, trigger.alphaIn)
		elseif ( trigger.lastFadeTime ~= nil ) then
			chosenTrigger = chosenTrigger or trigger
			
			local now = GetTime()
			local sinceTrue = now - trigger.lastFadeTime
			local speed = (trigger.alphaOut - trigger.alphaIn) / trigger.secondsOut
			local chosenSinceTrue = now - chosenTrigger.lastFadeTime
			local chosenSpeed = (chosenTrigger.alphaOut - chosenTrigger.alphaIn) / chosenTrigger.secondsOut
			
			if ( sinceTrue < chosenSinceTrue or (sinceTrue == chosenSinceTrue and speed < chosenSpeed) ) then
				chosenTrigger = trigger
			end
		end
	end
	
	if ( chosenTrigger ~= nil ) then
		local speed = (chosenTrigger.alphaOut - chosenTrigger.alphaIn) / chosenTrigger.secondsOut
		duration = (endAlpha - startAlpha) / speed
	end
	
	if ( Kong_Frames[frame].disableDrawWhenHidden and endAlpha == 0 ) then
		finishedFunc = Kong_DisableDraw
	end
	
	Kong_Fade(frame, startAlpha, endAlpha, duration, finishedFunc)
end

function Kong_FindGlobalIndex(Frame)

    local name = Frame:GetName();
    if name then
        return "[\"" .. name .. "\"]";
    end

    for i,x in pairs(_G) do
        if x == Frame then
            if type(i) == "string" then
                return "[\"" .. i .. "\"]";
            elseif type(i) == "number" or type(i) == "boolean" then
                return "[".. tostring(i) .. "]"
            end
        end
    end
    
    for _, Child in pairs({Frame:GetChildren()}) do
        name = Kong_FindGlobalIndex(Child);
        if name then
            return name..":GetParent()";
        end
    end
end

function Kong_CreateTrigger(Frame)

    local Trigger = {};
    setmetatable(Trigger, {__index = Kong_Frames[Frame].TriggerDefaults or TriggerDefaults});
    
    if Kong_Frames[Frame].TriggerDefaults then
        Trigger.refs = 0;
    else
        Trigger.Frame = Frame
    end
    
    return Trigger;
end

function Kong_RegisterFrame(region)
	Kong_Frames[region] = {}
	region.KongGetAlpha = region.GetAlpha
	region.KongSetAlpha = region.SetAlpha
	Kong_SetFader(region, false)
	local index = Kong_FindGlobalIndex(region)
	if ( index ~= nil ) then
		Kong_Frames[region].index = index
	else
		Kong_Frames[region].index = {}
		Kong_Warning(CANNOT_SAVE)
	end
	Kong_Frames[region].version = VERSION
	Kong_Frames[region].TriggerDefaults = Kong_CreateTrigger(region)
	Kong_Frames[region].Mouseover = Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create(MouseoverTypeName)))
	Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create("Unit in Combat")))
	Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create("Unit in Vehicle")))
end

function Kong_UnregisterFrame(region)
	for i = #Kong_Frames[region], 1, -1 do
		Kong_RemoveTrigger(region, Kong_Frames[region][i])
	end
	region.KongAlphaGroup:Stop()
	Kong_OverrideAlpha(region, false)
	Kong_Fade(region, 1, 1, 0)
	region.KongGetAlpha = nil
	region.KongSetAlpha = nil
	local index = Kong_Frames[region].index
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[index] ~= nil ) then
			group[index] = nil
			if ( next(group) == nil ) then
				Kong_Settings.Groups[name] = nil
			end
		end
	end
	Kong_Frames[region] = nil
end

function Kong_OnConditionStateChange(region, trigger)
	if ( trigger.Condition:IsMet() ) then
		Kong_FadeIn(region, trigger)
	else
		Kong_FadeOut(region, trigger)
	end
end

function Kong_AddTrigger(region, condition)
	local trigger = Kong_CreateTrigger(region)
	trigger.Condition = condition
	trigger.Condition:AddListener(Kong_OnConditionStateChange, region, trigger)
	table.insert(Kong_Frames[region], trigger)
	if ( trigger.Condition:IsMet() ) then
		Kong_FadeIn(region, trigger)
	end
	return trigger
end

function Kong_RemoveTrigger(region, trigger)
	if ( trigger.Condition:IsMet() ) then
		Kong_FadeOut(region, trigger)
	end
	trigger.Condition:RemoveListener(Kong_OnConditionStateChange)
	LibUSC:Deflate(trigger.Condition)
	for i, v in ipairs(Kong_Frames[region]) do
		if ( v == trigger ) then
			table.remove(Kong_Frames[region], i)
			break
		end
	end
end

function Kong_AddFrameToGroup(region, name)
	Kong_Settings.Groups[name] = Kong_Settings.Groups[name] or {}
	local group = Kong_Settings.Groups[name]
	local index = Kong_Frames[region].index
	if ( group[index] == nil ) then
		group[index] = region
		return true
	end
end

function Kong_RemoveFrameFromGroup(region, name)
	local group = Kong_Settings.Groups[name]
	local index = Kong_Frames[region].index
	if ( group ~= nil and group[index] ~= nil ) then
		group[index] = nil
		if ( next(group) == nil ) then
			Kong_Settings.Groups[name] = nil
		end
		return true
	end
end
  
function Kong_Enable()
    Kong_Settings.enabled = true;
    for Frame in pairs(Kong_Frames) do
        Kong_CalculateFadeOut(Frame);
    end
end

function Kong_Disable()
    for Frame in pairs(Kong_Frames) do
        if Kong_Frames[Frame].disableDrawWhenHidden and (Frame:KongGetAlpha() == 0) then
            Kong_EnableDraw(Frame)
        end
        Kong_Fade(Frame, 1, 1, 0)
    end
    Kong_Settings.enabled = false;
end

function Kong_RenameFrame(region, newIndex)
	local index = Kong_Frames[region].index
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[index] ~= nil ) then
			group[index] = nil
			group[newIndex] = region
		end
	end
	Kong_Frames[region].index = newIndex
end

function Kong_LoadSettings()

    -- Only need to initialize once at start and for new profiles
    if Kong_Initialized then
        return
    end
    Kong_Initialized = true
    
    -- Replace spaces in profiles created prior to v0.9c
    local TempProfiles = {};
    for n, p in pairs(Kong_Profiles) do 
        if string.find(n, " ") then
            TempProfiles[string.gsub(n," ","_")] = p;
            Kong_Profiles[n] = nil;
        end
    end
    for n, p in pairs(TempProfiles) do
        Kong_Profiles[n] = TempProfiles[n];
    end
    
    local defaultProfile = string.lower(UnitName("player"))
        .."_of_"..string.lower(string.gsub(GetRealmName()," ","_"));
    local enabled = true;
    
    -- Convert from previous version
    if Kong_Settings then
    
        local version = tonumber((Kong_Settings.version or "0"):gsub("%a",""),10)
        
        if version >= 0.7 and version < 0.9 then
            if not Kong_Settings.Groups then
                Kong_Settings.Groups = {};
            end
            
            -- TriggerDefaults.refs was being saved in version 0.7
            for _,Frame in pairs(Kong_Settings.Frames) do
                Frame.TriggerDefaults.refs = nil;
            end
            
            -- Convert current settings to a profile
            enabled = Kong_Settings.enabled;
            Kong_Settings.enabled = nil;
            Kong_Profiles[defaultProfile] = Kong_Settings;
        end
    end
    
    if (not Kong_Settings) or (not Kong_Settings.profile) then
        Kong_Settings = {
            profile = defaultProfile,
            enabled = enabled,
        };
    end
    Kong_Settings.profile = string.gsub(Kong_Settings.profile," ","_");
        
    -- If creating a new profiles
    if not Kong_Profiles[Kong_Settings.profile] then
        Kong_Profiles[Kong_Settings.profile] = {
            Frames = {},
            Groups = {},
        };
    end
    
    -- Kong_Settings will have all the same fields as the profile
    setmetatable(Kong_Settings, {__index = Kong_Profiles[Kong_Settings.profile]});
    
    -- Index frames by number (for pre-v1.1 compatibility)
    local tempFrames = Kong_Settings.Frames;
    Kong_Profiles[Kong_Settings.profile].Frames = {}
    for _, SavedFrame in pairs(tempFrames) do
        table.insert(Kong_Settings.Frames, SavedFrame)
    end
    
    -- Initialize group-frames to 0 (for pre-v1.1 compatibility)
    for _,Group in pairs(Kong_Settings.Groups) do
        for index in pairs(Group) do
            Group[index] = 0;
        end
    end
    
    -- Convert pre-v1.2 indexes
    for _, SavedFrame in pairs(Kong_Settings.Frames) do
        if string.sub(SavedFrame.index, 1, 1) ~= "[" then
            SavedFrame.index = gsub(SavedFrame.index, "%.", "[\"")
            SavedFrame.index = gsub(SavedFrame.index, "%[", "\"][")
            SavedFrame.index = gsub("[\""..SavedFrame.index.."\"]", "%]\"%]", "]")
        end
    end
    for groupName, Group in pairs(Kong_Settings.Groups) do
        local tempGroup = Group
        Kong_Settings.Groups[groupName] = {}
        for index in pairs(tempGroup) do
            if string.sub(index, 1, 1) ~= "[" then
                index = gsub(index, "%.", "[\"")
                index = gsub(index, "%[", "\"][")
                index = gsub("[\""..index.."\"]", "%]\"%]", "]")
            end
            Kong_Settings.Groups[groupName][index] = 0;
        end
    end
    
    for _, SavedFrame in pairs(Kong_Settings.Frames) do 
    
        -- Convert pre-v1.3 triggers
        for _, triggerName in ipairs({"mouseover", "combat", "nocombat", "vehicle", "novehicle", 
                                      "casting", "nocasting", "macro"}) do
        
            local Trigger = SavedFrame[triggerName]
            if Trigger then
                if "mouseover" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Frame has Mouse Focus"))
                elseif "combat" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Unit in Combat"))
                elseif "nocombat" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Not",
                            LibUSC:Create("Unit in Combat")))
                elseif "vehicle" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Unit in Vehicle"))
                elseif "novehicle" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Not",
                            LibUSC:Create("Unit in Vehicle")))
                elseif "casting" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Unit is Casting"))
                elseif "nocasting" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Not",
                            LibUSC:Create("Unit is Casting")))
                elseif "macro" == triggerName then
                    Trigger.Condition = LibUSC:Create("Condition", 
                        LibUSC:Create("Macro",
                            LibUSC:Create("User-Entered Text", Trigger.condition)))
                    Trigger.condition = nil
                end
                tinsert(SavedFrame, Trigger)
                SavedFrame[triggerName] = nil
            end
            
            Trigger = SavedFrame.health
            if Trigger then
                for unit in pairs(Trigger.Units) do
                    local TriggerCopy = CopyTable(Trigger)
                    if Trigger.lessThan then
                        TriggerCopy.Condition = LibUSC:Create("Condition", 
                            LibUSC:Create("Less Than",
                                LibUSC:Create("Unit Health (Percent)",
                                    LibUSC:Create("Convert Text to Unit",
                                        LibUSC:Create("User-Entered Text", unit))),
                                LibUSC:Create("User-Entered Number", Trigger.threshold)))
                    else
                        TriggerCopy.Condition = LibUSC:Create("Condition", 
                            LibUSC:Create("Greater Than",
                                LibUSC:Create("Unit Health (Percent)",
                                    LibUSC:Create("Convert Text to Unit",
                                        LibUSC:Create("User-Entered Text", unit))),
                                LibUSC:Create("User-Entered Number", Trigger.threshold)))
                    end
                    TriggerCopy.Units = nil
                    TriggerCopy.lessThan = nil
                    TriggerCopy.threshold = nil
                    tinsert(SavedFrame, TriggerCopy)
                end
                SavedFrame.health = nil
            end
        end
    end
    
    Kong_Profiles[Kong_Settings.profile].version = VERSION
    
    -- Fix for minimap fading
    if not MinimapParent then 
        MinimapParent = Minimap:GetParent()
        if MinimapParent ~= UIParent then
            MinimapParent:EnableMouse(true)
        
            MinimapParent.KongAlphaGroup = MinimapParent:CreateAnimationGroup()
            MinimapParent.KongAlphaAnimation = MinimapParent.KongAlphaGroup:CreateAnimation("Alpha")
            MinimapParent.KongAlphaAnimation:SetScript("OnPlay", function(self) 
                if self:GetToAlpha() - self:GetFromAlpha() > 0 then
                    Minimap:Show()
                end
            end)
            local function OnMinimapFinish(self)
                if MinimapParent:GetAlpha() == 0 then
                    Minimap:Hide()
                else
                    Minimap:Show()
                    
                    -- Keep minimap contents visible indoors (only works at 100% alpha)
                    local zoom = Minimap:GetZoom()
                    Minimap:SetZoom(mod(zoom+1,6))
                    Minimap:SetZoom(zoom)
                end
            end
            MinimapParent.KongAlphaAnimation:SetScript("OnFinished", OnMinimapFinish)
            MinimapParent.KongAlphaAnimation:SetScript("OnStop", OnMinimapFinish)
        end
    end

    Kong_LoadFrames()
    
    if not Kong_Settings.enabled then
       Kong_Warning("Kong is currently disabled. Use |cFF33FF99/kong toggle|r to enable it.")
    end
end

function Kong_LoadFrames()

    for i=#Kong_Settings.Frames,1,-1 do

        local SavedFrame = Kong_Settings.Frames[i]
        
        -- Convert stored path to frame
        if not SavedFrame.loader then
            SavedFrame.loader = loadstring("return _G"..SavedFrame.index)
        end
        
        local success, Root = pcall(SavedFrame.loader)
        
        if ( success and Root 
			and type(Root.IsObjectType == "function") and Root:IsObjectType("Region") ) then -- make sure it is a region; ticket #5

            -- Find the root frame for frames before version 1.6
            if tonumber((SavedFrame.version or "0"):gsub("%a",""),10) < 1.6 then
                            
                local Parent = Root:GetParent();
                while Parent and Parent ~= UIParent do
                    Root = Parent;
                    SavedFrame.index = SavedFrame.index..":GetParent()"
                    Parent = Parent:GetParent();
                end
            end
            SavedFrame.version = VERSION
            
            if not Kong_Frames[Root] then
                
                SavedFrame.loader = nil
                Kong_Frames[Root] = SavedFrame;
                table.remove(Kong_Settings.Frames, i)
                
                setmetatable(SavedFrame.TriggerDefaults, {__index = TriggerDefaults});
                SavedFrame.TriggerDefaults.Frame = Root
                
                Root.KongGetAlpha = Root.GetAlpha
                Root.KongSetAlpha = Root.SetAlpha
                if SavedFrame.overrideAlpha then
                    Kong_OverrideAlpha(Root, true)
                end
                
                Kong_SetFader(Root, SavedFrame.compatibilityFader)
                
                -- Initialize each trigger
                for _,Trigger in ipairs(SavedFrame) do
                    setmetatable(Trigger, {__index = SavedFrame.TriggerDefaults});
                    Trigger.refs = 0;
                    LibUSC:Inflate(Trigger.Condition)
                    Trigger.Condition:AddListener(Kong_OnConditionStateChange, Root, Trigger)
                    if MouseoverTypeName == Trigger.Condition.Parameters[1]:GetName() then
                        SavedFrame.Mouseover = Trigger
                    end
                end
                
                if Kong_Settings.enabled then
                    Kong_CalculateFadeOut(Root);
                end
                
                -- Fade frame according to current state
                for _,Trigger in ipairs(SavedFrame) do
                    if Trigger.Condition:IsMet() then
                        Kong_FadeIn(Root, Trigger);
                    end
                end

                -- Connect the mouseover group list to actual frames
                for groupName,Group in pairs(Kong_Settings.Groups) do
                    if Group[SavedFrame.index] then
                    
                        -- Fade in the frame if the group is active
                        for i=1,Group[SavedFrame.index] do
                            Kong_FadeIn(Root, SavedFrame.Mouseover);
                        end
                        
                        Group[SavedFrame.index] = Root;
                    end
                end
            end
        end
    end
end

function Kong_Unload()

    for Root, Settings in pairs(Kong_Frames) do

        if Kong_Frames[Root].disableDrawWhenHidden and (Root:KongGetAlpha() == 0) then
            Kong_EnableDraw(Root)
        end
            
        Root.KongAlphaGroup:Stop()
        Kong_OverrideAlpha(Root, false)
        Kong_Fade(Root, 1, 1, 0)
        Root.KongGetAlpha = nil
        Root.KongSetAlpha = nil
        
        -- Remove runtime-specific data
        for _,Trigger in ipairs(Settings) do
            Trigger.refs = nil
            Trigger.lastFadeTime = nil
            Trigger.Condition:RemoveListener(Kong_OnConditionStateChange)
            LibUSC:Deflate(Trigger.Condition)
        end
        Settings.loader = nil
        Settings.Mouseover = nil
        Settings.TriggerDefaults.Frame = nil
        
        -- Move it back to Kong_Settings to be saved
        Kong_Frames[Root] = nil;
        if type(Settings.index) ~= "table" then
            table.insert(Kong_Settings.Frames, Settings);
        end
    end
    
    if MinimapParent then
        MinimapParent.KongAlphaAnimation:GetScript("OnFinished")()
    end
    
    -- "disconnect" the groups from the actual frames
    for _,Group in pairs(Kong_Settings.Groups) do
        for index in pairs(Group) do
            Group[index] = 0
        end
    end
        
    -- In case a new profile is about to be loaded
    Kong_Frames = {};
	lastMouseFocus = nil
	lastMouseStart = nil
    Kong_Initialized = false;
end

function Kong_ProfileCommandHandler(args)

    operation = args[2];
    if #args > 2 then
        profile = table.concat(args, "_", 3);
    end
    
    if not operation then
    
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_COMMANDS));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_LOAD));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_CREATE));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_DELETE));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_LIST));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_CURRENT));
        DEFAULT_CHAT_FRAME:AddMessage(string.format(HELP_PROFILE_RESET));
    
    elseif operation == "load" then
    
        if profile then
            if Kong_Profiles[profile] then
                if profile ~= Kong_Settings.profile then
                
                    Kong_Unload();
                    Kong_Settings.profile = profile;
                    Kong_LoadSettings();
                    
                    Kong_Info(PROFILE_LOADED, profile);
                else
                    Kong_Error(PROFILE_ALREADY_LOADED, profile);
                end
            else
                Kong_Error(NO_SUCH_PROFILE, profile);
            end
        else
            Kong_Error(PROFILE_NAME_REQUIRED);
        end
    elseif operation == "create" then
    
        if profile then
            if not Kong_Profiles[profile] then
            
                Kong_Unload();
                Kong_Settings.profile = profile;
                Kong_LoadSettings();
                
                Kong_Info(NOW_USING_PROFILE, profile);
            else 
                Kong_Error(PROFILE_ALREADY_EXISTS, profile);
            end
        else
            Kong_Error(PROFILE_NAME_REQUIRED);
        end
    elseif operation == "list" then
        local ProfileNames = {};
        for name in pairs(Kong_Profiles) do
            tinsert(ProfileNames, name);
        end
        Kong_Info("Available profiles: "..table.concat(ProfileNames, ", "));
        
    elseif operation == "current" then
    
        Kong_Info("Current profile is: "..Kong_Settings.profile);
        
    elseif operation == "delete" then
    
        if profile then
            if profile ~= Kong_Settings.profile then
                if Kong_Profiles[profile] then
                    Kong_Profiles[profile] = nil;
                    Kong_Info(PROFILE_DELETED, profile);
                else
                    Kong_Error(NO_SUCH_PROFILE, profile);
                end
            else
                Kong_Error(CANNOT_DELETE_CURRENT_PROFILE);
            end
        else
            Kong_Error(PROFILE_NAME_REQUIRED);
        end
    elseif operation == "reset" then
        Kong_ResetCommandHandler()
    else
        Kong_Error(UNKNOWN_OPTION, operation, args[1]);
    end
end

function Kong_ResetCommandHandler()
    StaticPopup_Show("KONG_RESET", Kong_Settings.profile)
end

function Kong_EnableCommandHandler()
    if not Kong_Settings.enabled then
        Kong_Enable()
        Kong_Info(ENABLED);
    else
        Kong_Error(ALREADY_ENABLED);
    end
end

function Kong_DisableCommandHandler()
    if Kong_Settings.enabled then
        Kong_Disable()
        Kong_Info(DISABLED);
    else
        Kong_Error(ALREADY_DISABLED);
    end
end

function Kong_ToggleCommandHandler()
    if Kong_Settings.enabled then
        Kong_DisableCommandHandler();
    else
        Kong_EnableCommandHandler();
    end
end

function Kong_CommandHandler(args)
    local args = {strsplit(" ", string.lower(args))};
    local command = args[1];

    if command and command ~= "" and KongConfig and KongConfig.frame:IsShown() then 
        Kong_Error("The config screen must be closed to issue /kong commands.")
    elseif command == "enable" then
        Kong_EnableCommandHandler();
    elseif command == "disable" then
        Kong_DisableCommandHandler();
    elseif command == "toggle" then
        Kong_ToggleCommandHandler();
    elseif command == "profile" then
        Kong_ProfileCommandHandler(args);
    elseif command == "reset" then
        Kong_ResetCommandHandler(args)
    elseif command and command ~= "" and command ~= "help" then
        Kong_Error("\""..command.."\""..INVALID_COMMAND);
    elseif command ~= "help" and LoadAddOn('KongConfig') then
            KongConfig_Show()
    else
        if command ~= "help" then
            Kong_Warning(NO_KONG_CONFIG);
        end
        DEFAULT_CHAT_FRAME:AddMessage(HELP_COMMANDS);
        DEFAULT_CHAT_FRAME:AddMessage(HELP_TOGGLE);
        DEFAULT_CHAT_FRAME:AddMessage(HELP_PROFILE);
    end
end
SLASH_KONG1 = "/kong";
SlashCmdList["KONG"] = Kong_CommandHandler;

local function FindFirstTrackedRegion(region)
	while ( region ~= nil and Kong_Frames[region] == nil ) do
		region = not region:IsForbidden() and region:GetParent() or nil
	end
	return region
end

local function ForEachTrackedRegion(region, func, ...)
	while ( region ~= nil ) do
		if ( Kong_Frames[region] ~= nil ) then func(region, ...) end
		region = not region:IsForbidden() and region:GetParent() or nil
	end
end

local function MouseoverRegionFader(region, func, additive)
	local settings = Kong_Frames[region]
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[settings.index] ~= nil ) then
			for index, groupRegion in pairs(group) do
				if ( type(groupRegion) == "table" ) then -- TODO: remove value-type ambiguity
					func(groupRegion, Kong_Frames[groupRegion].Mouseover)
				else
					group[index] = groupRegion + additive
				end
			end
		end
	end
	func(region, settings.Mouseover)
end

local FRAME_FIND_PERIOD = 1.0
local frameFindTimer = 0

function Kong_OnUpdate(self, elapsed)
    frameFindTimer = frameFindTimer - elapsed
    if frameFindTimer <= 0 then
        frameFindTimer = FRAME_FIND_PERIOD
        Kong_LoadFrames()
    end
	
	local mouseFocus = GetMouseFocus()
	if ( mouseFocus ~= lastMouseFocus ) then
		local mouseStart = mouseFocus ~= nil and FindFirstTrackedRegion(mouseFocus) or nil
		if ( mouseStart ~= lastMouseStart ) then
			if ( mouseStart ~= nil ) then
				ForEachTrackedRegion(mouseStart, MouseoverRegionFader, Kong_FadeIn, 1)
			end
			if ( lastMouseStart ~= nil ) then
				ForEachTrackedRegion(lastMouseStart, MouseoverRegionFader, Kong_FadeOut, -1)
			end
			lastMouseStart = mouseStart
		end
		lastMouseFocus = mouseFocus
	end
end

function Kong_OnEvent(self, event, arg1, ...)

    if event == "ADDON_LOADED" then
        if arg1 == "Kong" and not Kong_Settings then 
            Kong_Info(WELCOME)
        end
    elseif event == "PLAYER_LOGIN" then
	
		-- SexyMap custom-position workaround
		if ( IsAddOnLoaded("SexyMap") and Minimap:GetParent() == UIParent ) then
			CreateFrame("frame", "MinimapWrapper", UIParent)
			Minimap:SetParent(MinimapWrapper)
			MinimapWrapper:SetAllPoints(Minimap)
		end
		
        Kong_LoadSettings();
        hooksecurefunc("CreateFrame", function() frameFindTimer = 0; end)
        KongUIFader:SetScript("OnUpdate", Kong_OnUpdate);
    elseif event == "PLAYER_LOGOUT" then
        xpcall(Kong_Unload, function(message) Kong_Settings.error = message.."\n"..debugstack() end)
    end
end

-- Fix for TukUI PetBattleHider -- TODO remove after v1.6
if TukuiPetBattleHider then 
    for _,TukChild in pairs({TukuiPetBattleHider:GetChildren()}) do
        TukChild:SetParent(UIParent)
        local unit = TukChild:GetAttribute("unit")
        if unit then
            RegisterUnitWatch(TukChild, true)
            RegisterStateDriver(TukChild, "visibility", "[@"..unit..",exists,nopetbattle] show; hide")
        else
            RegisterStateDriver(TukChild, "visibility", "[nopetbattle] show; hide")
        end
    end
    UnregisterStateDriver(TukuiPetBattleHider, "visibility")
    TukuiPetBattleHider:Hide()
    TukuiPetBattleHider = UIParent
end

-- The event receiver frame
KongUIFader = CreateFrame("Frame");
KongUIFader:SetScript("OnEvent", Kong_OnEvent);

KongUIFader:RegisterEvent("ADDON_LOADED");
KongUIFader:RegisterEvent("PLAYER_LOGIN");
KongUIFader:RegisterEvent("PLAYER_LOGOUT");
